//
//  TabBarContentView.h
//  TabBarContentView
//
//  Created by Vivitickey on 2017/5/17.
//  Copyright © 2017年 test. All rights reserved.
//

#import <UIKit/UIKit.h>

@class TabBarContentView ;

@protocol TabBarContentViewDataSource <NSObject>
@required
/**
 返回当前控件Item的个数
 
 @param tabBarContentView 当前控件
 @return 返回选项个数
 */
- (NSInteger)numberOfItemsInTabBarContentView:(TabBarContentView *)tabBarContentView;

/**
 提供选项的标题
 
 @param tabBarContentView 当前控件
 @param index 标题索引
 @return 返回标题内容
 */
- (NSString *)tabBarContentView:(TabBarContentView *)tabBarContentView titleForItemAtIndex:(NSInteger)index;

/**
 提供内容视图
 
 @param tabBarContentView 当前控件
 @param index 标题索引
 @return 内容视图
 */
- (UIView *)tabBarContentView:(TabBarContentView *)tabBarContentView contentViewAtIndex:(NSInteger)index;


@end

@protocol TabBarContentViewDelegate <NSObject>


@optional
/**
 定义导航条的高度,如果不定义默认 30
 
 @param tabBarContentView 当前控件
 @return 高度
 */
- (CGFloat)heightForTabBarInTabBarContentView:(TabBarContentView *)tabBarContentView;


/**
 设置标题的颜色, 默认黑色
 
 @param tabBarContentView 当前控件
 @return 标题颜色
 */
- (UIColor *)colorForTabBarItemTextInTabBarContentView:(TabBarContentView *)tabBarContentView;

/**
 当前选中的颜色,默认值: 橘黄色
 
 @param tabBarContentView 当前控件
 @return 返回选中的颜色
 */
- (UIColor *)highlightColorForTabBarItemInTabBarContentView:(TabBarContentView *)tabBarContentView;


/**
 返回一个当前选项选中的视图
 
 @param tabBarContentView 当前控件
 @return 选中的视图
 */
- (UIView *)highlightViewForTabBarItemInTabBarContentView:(TabBarContentView *)tabBarContentView;


/**
 当选中状态发生改变时回调
 
 @param tabBarContentView 当前控件
 @param index 选项下标
 */
- (void)tabBarContentView:(TabBarContentView *)tabBarContentView didSelectItemAtIndex:(NSInteger)index;

@end


@interface TabBarContentView : UIView

/**
 代理方法
 */
@property (weak,nonatomic) id<TabBarContentViewDelegate>        delegate;

/**
 数据源方法
 */
@property (weak,nonatomic) id<TabBarContentViewDataSource>      dataSource;

@property (strong,nonatomic) UIView                             *tabBarBackgroundView;




/**
 刷新界面
 */
-(void)reloadData;


@end
