//
//  TabBarContentView.m
//  TabBarContentView
//
//  Created by Vivitickey on 2017/5/17.
//  Copyright © 2017年 test. All rights reserved.
//

#import "TabBarContentView.h"
#import <Masonry/Masonry.h>

@interface TabBarContentView ()<UIScrollViewDelegate>

/**
 导航条
 */
@property (strong, nonatomic)   UIScrollView    *tabBarScrollView;

/**
 内容
 */
@property (strong, nonatomic)   UIScrollView    *contentScrollView;


/**
 存放 TabBarItem 容器
 */
@property (strong, nonatomic)   NSMutableArray<UIButton *>  *tabBarItems;

@property (strong, nonatomic)   NSMutableArray<UIView *>    *contents;

@property (strong, nonatomic)   UIButton                    *selectedItemButton;

/**
 用于存选中视图的容器
 */
@property (strong, nonatomic)   UIView                      *highlightViewContainer;

@end

@implementation TabBarContentView{
    
    NSInteger _count;
    
}

#pragma mark - 重写属性 -
-(UIScrollView *)tabBarScrollView{
    //创建tabbar空间
    if (_tabBarScrollView == nil) {
        _tabBarScrollView = [UIScrollView new];
        _tabBarScrollView.pagingEnabled = YES;
        _tabBarScrollView.showsVerticalScrollIndicator = NO;
        _tabBarScrollView.showsHorizontalScrollIndicator = NO;
        [self addSubview:_tabBarScrollView];
        //默认布局
        [_tabBarScrollView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.edges.equalTo(self);
        }];
    }
    return _tabBarScrollView;
}

-(UIScrollView *)contentScrollView{
    if (_contentScrollView == nil) {
        _contentScrollView = [UIScrollView new];
        _contentScrollView.pagingEnabled = YES;
        _contentScrollView.showsHorizontalScrollIndicator = NO;
        _contentScrollView.showsVerticalScrollIndicator = NO;
        _contentScrollView.delegate = self;
        [self addSubview:_contentScrollView];
        [_contentScrollView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.equalTo(self);
            make.top.equalTo(self.tabBarScrollView.mas_bottom);
        }];
    }
    return _contentScrollView;
}

-(NSMutableArray<UIView *> *)contents{
    if (_contents == nil) {
        _contents = [NSMutableArray array];
    }
    return _contents;
}

-(NSMutableArray<UIButton *> *)tabBarItems {
    if(_tabBarItems == nil) {
        _tabBarItems = [NSMutableArray array];
    }
    return _tabBarItems;
}

-(void)setTabBarBackgroundView:(UIView *)tabBarBackgroundView{
    // 设置背景视图
    if (_tabBarBackgroundView != nil) {
        [_tabBarBackgroundView removeFromSuperview];
        _tabBarBackgroundView = nil;
    }
    
    _tabBarBackgroundView = tabBarBackgroundView;
    
    [self.tabBarScrollView insertSubview:_tabBarBackgroundView atIndex:0];
    [_tabBarBackgroundView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.with.height.equalTo(self.tabBarScrollView);
    }];
}

#pragma mark - 私有方法 -
-(CGFloat)tabBarHeight{
    //获取tabBar高度
    CGFloat tabBarHeight = 30;
    if (self.delegate && [self.delegate respondsToSelector:@selector(heightForTabBarInTabBarContentView:)]) {
        tabBarHeight = [self.delegate heightForTabBarInTabBarContentView:self];
    }
    return tabBarHeight;
}

-(NSInteger)numberOfItems{
    if (self.dataSource &&[self.delegate respondsToSelector:@selector(numberOfItemsInTabBarContentView:)]) {
        return [self.dataSource numberOfItemsInTabBarContentView:self];
    }
    return 0;
}

/**
 刷新导航条
 */
-(void)reloadTabBar{
    //刷新布局
    [self.tabBarScrollView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.equalTo(self);
        make.height.mas_equalTo([self tabBarHeight]);
    }];
    //清空TabBar内部的内容
    for (UIButton *itemButton in self.tabBarItems) {
        [itemButton removeFromSuperview];
    }
    [self.tabBarItems removeAllObjects];
    
    //重新构建TabBar选项
    //获取数量
    NSInteger tabBarItemCount = _count;
    //构建
    for (NSInteger i = 0; i < tabBarItemCount; i++) {
        UIButton *itemButton = [UIButton buttonWithType:UIButtonTypeCustom];
        [self.tabBarScrollView addSubview:itemButton];
        
        [itemButton addTarget:self action:@selector(actionSelectItemButton:) forControlEvents:UIControlEventTouchUpInside];
        
        //设置标题
        if (self.dataSource && [self.dataSource respondsToSelector:@selector(tabBarContentView:titleForItemAtIndex:)]) {
            NSString *title = [self.dataSource tabBarContentView:self titleForItemAtIndex:i];
            [itemButton setTitle:title forState:UIControlStateNormal];
        }
        // 设置标题颜色
        if(self.delegate && [self.delegate respondsToSelector:@selector(colorForTabBarItemTextInTabBarContentView:)]) {
            [itemButton setTitleColor:[self.delegate colorForTabBarItemTextInTabBarContentView:self] forState:UIControlStateNormal];
        } else {
            [itemButton setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
        }
        // 设置选中的颜色
        if(self.delegate && [self.delegate respondsToSelector:@selector(highlightColorForTabBarItemInTabBarContentView:)]) {
            [itemButton setTitleColor:[self.delegate highlightColorForTabBarItemInTabBarContentView:self] forState:UIControlStateSelected];
        } else {
            [itemButton setTitleColor:[UIColor orangeColor] forState:UIControlStateSelected];
        }
        
        [self.tabBarItems addObject:itemButton];
    }
    
    //布局
    CGFloat percentWidth = 1.0 / tabBarItemCount;
    for (NSInteger i = 0; i < tabBarItemCount; i++) {
        UIButton *itemButton = self.tabBarItems[i];
        [itemButton mas_makeConstraints:^(MASConstraintMaker *make) {
            if (i == 0) {
                make.left.equalTo(self.tabBarScrollView.mas_left);
            } else {
                make.left.equalTo(self.tabBarItems[i - 1].mas_right);
            }
            
            if (i == tabBarItemCount - 1) {
                make.right.equalTo(self.tabBarScrollView.mas_right);
            } else {
                make.right.equalTo(self.tabBarItems[i + 1].mas_left);
            }
            
            make.top.bottom.height.equalTo(self.tabBarScrollView);
            make.width.equalTo(self.tabBarScrollView).multipliedBy(percentWidth);
        }];
    }
    
    //构建选中的视图模型
    if(self.highlightViewContainer == nil){
        self.highlightViewContainer = [UIView new];
        self.highlightViewContainer.userInteractionEnabled = NO;
        [self.tabBarScrollView addSubview:self.highlightViewContainer];
        [self.highlightViewContainer mas_makeConstraints:^(MASConstraintMaker *make) {
            make.edges.equalTo(self.tabBarItems.firstObject);
        }];
    }
    
    NSArray *subviews = [self.highlightViewContainer subviews];
    for (UIView *subview in subviews) {
        [subview removeFromSuperview];
    }
    
    if (self.delegate && [self.delegate respondsToSelector:@selector(highlightViewForTabBarItemInTabBarContentView:)]) {
        UIView *highlightView = [self.delegate highlightViewForTabBarItemInTabBarContentView:self];
        [self.highlightViewContainer addSubview:highlightView];
        [highlightView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.edges.equalTo(self.highlightViewContainer);
        }];
    }
    
    [self actionSelectItemButton:self.tabBarItems.firstObject];
    
}

/**
 刷新内容
 */
- (void)realoadContent {
    // 清空老的内容
    for (UIView *contentView in self.contents) {
        [contentView removeFromSuperview];
    }
    [self.contents removeAllObjects];
    
    
    
    // 构建内容视图
    for (NSInteger i = 0; i < _count; i++) {
        UIView *contentView = nil;
        if(self.dataSource && [self.dataSource respondsToSelector:@selector(tabBarContentView:contentViewAtIndex:)]) {
            contentView = [self.dataSource tabBarContentView:self contentViewAtIndex:i];
        }
        if(contentView == nil) {
            contentView = [UIView new];
        }
        
        [self.contentScrollView addSubview:contentView];
        [self.contents addObject:contentView];
    }
    
    // 布局内容视图
    for (NSInteger i = 0; i < _count; i++) {
        UIView *contentView = self.contents[i];
        [contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            if(i == 0) {
                make.left.equalTo(self.contentScrollView.mas_left);
            } else {
                make.left.equalTo(self.contents[i - 1].mas_right);
            }
            if(i == _count - 1) {
                make.right.equalTo(self.contentScrollView.mas_right);
            } else {
                make.right.equalTo(self.contents[i + 1].mas_left);
            }
            
            make.top.height.bottom.width.equalTo(self.contentScrollView);
            
        }];
    }
}
#pragma mark - 动作方法 -

/**
 选择选中的Button
 
 @param button 选中的Button
 */
-(void)actionSelectItemButton:(UIButton *)button {
    
    // 过滤
    if(self.selectedItemButton != nil && self.selectedItemButton == button) return;
    
    
    if(self.selectedItemButton != nil) {
        self.selectedItemButton.selected = NO;
        self.selectedItemButton = nil;
    }
    self.selectedItemButton = button;
    self.selectedItemButton.selected = YES;
    
    // 联动内容
    NSInteger selectedIndex = [self.tabBarItems indexOfObject:self.selectedItemButton];
    
    if(self.delegate && [self.delegate respondsToSelector:@selector(tabBarContentView:didSelectItemAtIndex:)]) {
        [self.delegate tabBarContentView:self didSelectItemAtIndex:selectedIndex];
    }
    
    
    CGPoint offset = CGPointMake(selectedIndex * self.contentScrollView.frame.size.width, 0);
    [self.contentScrollView setContentOffset:offset animated:YES];
}

#pragma mark - 公共方法 -
/**
 刷新界面
 */
-(void)reloadData{
    
    _count = [self numberOfItems];
    
    [self reloadTabBar];
    [self realoadContent];
    
}

#pragma mark - UIScrollViewDelegate -
-(void)scrollViewDidScroll:(UIScrollView *)scrollView{
    
    // 计算选中 TabBarItem
    CGFloat offsetX = scrollView.contentOffset.x / (scrollView.contentSize.width - scrollView.frame.size.width) * (self.tabBarScrollView.frame.size.width - self.highlightViewContainer.frame.size.width);
    
    // 仅仅设置了frame
    self.highlightViewContainer.frame = CGRectMake(offsetX, self.highlightViewContainer.frame.origin.y, self.highlightViewContainer.frame.size.width, self.highlightViewContainer.frame.size.height);
    
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    
    NSInteger selectedIndex = scrollView.contentOffset.x / scrollView.frame.size.width;
    UIButton *selectedButton = self.tabBarItems[selectedIndex];
    [self actionSelectItemButton:selectedButton];
    
    
    // 当滚动完成以后更新高亮容器视图的约束
    [self.highlightViewContainer mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.selectedItemButton);
    }];
}
@end
